import processing.core.*; 
import processing.xml.*; 

import krister.Ess.*; 
import sojamo.drop.*; 
import controlP5.*; 

import java.applet.*; 
import java.awt.*; 
import java.awt.image.*; 
import java.awt.event.*; 
import java.io.*; 
import java.net.*; 
import java.text.*; 
import java.util.*; 
import java.util.zip.*; 
import java.util.regex.*; 

public class FractalWavetables extends PApplet {


/**
 * Drag the mouse over the pattern sliders to create sound. 
 * Add or remove sliders via the "steps" control.
 * <br/><br/>Try setting the number of steps to an interesting metrical unit &#8212; say "8" &#8212; 
 * and use the pattern as a fractal step sequencer.
 * <br/><br/>If the UI freezes, try reloading the page. 
 * <br/><br/>If you like the sounds, you can download and run the app locally to save out files. 
 * <p><a href="http://www.raintone.com/code/processing/FractalWavetables/application.macosx.zip">Mac OS X version</a><br />
 * <a href="http://www.raintone.com/code/processing/FractalWavetables/application.windows.zip">Windows version</a><br />
 * <a href="http://www.raintone.com/code/processing/FractalWavetables/application.linux.zip">Linux version</a></p>
 * <a href="http://www.raintone.com/">Back to Raintone.com</a>
 * <br/></br>
 * <br/><br/>Thanks to Terran Olson's work on <a href="http://www.halfcadence.net/audio-fractals/">audio fractals</a> that inspired this sketch.
 * Thanks to Krister Olsson's <a href="http://www.tree-axis.com/Ess/">Ess library</a> for the sound support, and Andreas Schlegel's <a href="http://www.sojamo.de/libraries/controlP5/">controlP5 library</a> for UI.
 * <br/><br/>
 */

// Fractal Wavetables
// March 2009
// jdn (at) raintone.com
//
// change history:
// v1 - basic algorithm and mono saving
// v2 - added drag-and-drop import of fw_xxxx.aif files
// v3 - code cleanup
// v4 - switched to controlP5 library for most of GUI.
//      refactored code -- FWAudio could become class
// v5 - added a, b, stereo a+b and morph a->b modes
//
// 

  // nice simple sound library

  // holy shit this library rocks


/*
 * Model
 */

// Important constants
final int SR = 44100;
final int MAX_SLIDERS = 80;
final int LEFT_MARGIN = 100;

// fractal data
final int NUM_PATTERNS = 2;
FloatFract[] fract = new FloatFract[NUM_PATTERNS];  // to support stereo
int numPatternsActive = 1;
int patternOffset = 1;
boolean morphing = false;

boolean update = true;

// pattern size / timing data
float curDuration = 0;
int targetIteration = 0;

FWAudio playback;

/*
 * Control
 */

SDrop drop;  // for dropping saved files back into the program to load as "presets"
  
public ControlP5 controlP5;
Slider stepsSlider;
Slider durationSlider;
FWSliderPool[] patternSliders = new FWSliderPool[NUM_PATTERNS];
IterationView[] fractView = new IterationView[NUM_PATTERNS];

ArrayList mouseListeners = new ArrayList();

// create and wire the entire GUI...
public void setup() {
  Ess.start(this);
  
  playback = new FWAudio();

  // drag-and-drop handler for file imports
  drop = new SDrop(this);

  size(800, 600);
  colorMode(HSB, 1);
  background(0);
  
  // setup UI controller
  controlP5 = new ControlP5(this);
  
  // mode buttons
  int buttonWidth = 77;
  int buttonHeight = 20;
  int buttonY = 14;
  int b = 1;
  controlP5.addButton("a",1,LEFT_MARGIN,buttonY,buttonWidth,buttonHeight);
  controlP5.addButton("b",1,LEFT_MARGIN+(b++)*(buttonWidth+10),buttonY,buttonWidth,buttonHeight);
  controlP5.addButton("stereo",1,LEFT_MARGIN+(b++)*(buttonWidth+10),buttonY,buttonWidth,buttonHeight).setLabel("stereo");
  controlP5.addButton("morph",1,LEFT_MARGIN+(b++)*(buttonWidth+10),buttonY,buttonWidth,buttonHeight);
  controlP5.addButton("swap",1,LEFT_MARGIN+(b++)*(buttonWidth+10),buttonY,buttonWidth,buttonHeight).setLabel("swap a<->b");
  // save button -- only if we're running as an application
  if(!online) {
    controlP5.addButton("save",1,LEFT_MARGIN+(b++)*(buttonWidth+10),buttonY,buttonWidth,buttonHeight).setLabel("Save Audio");
  }
  
  // play toggles
  buttonWidth = 20;
  buttonHeight = 20;
  buttonY = 300;
  b = 0;
  controlP5.addToggle("mute",false,LEFT_MARGIN+(b++)*(buttonWidth+10),buttonY,buttonWidth,buttonHeight);
    controlP5.addToggle("looping",true,LEFT_MARGIN+(b++)*(buttonWidth+10),buttonY,buttonWidth,buttonHeight).setLabel("loop");
  controlP5.addToggle("update",true,LEFT_MARGIN+(1+b++)*(buttonWidth+10),buttonY,buttonWidth,buttonHeight).setLabel("autoupdate");
  
  // horizontal sliders
  stepsSlider = controlP5.addSlider("steps",2,MAX_SLIDERS,3,LEFT_MARGIN,245,width-200,10);
  stepsSlider.setLabel("");
  durationSlider = controlP5.addSlider("duration",0.25f,30,4.0f,LEFT_MARGIN,275,width-200,10); 
  durationSlider.setLabel("seconds");
  durationSlider.setDecimalPrecision(2);  
  
  // labels
  controlP5.addTextlabel("patternLabel", "pattern", LEFT_MARGIN-43, 121);
  controlP5.addTextlabel("stepsLabel", "steps", LEFT_MARGIN-33, 246);
  controlP5.addTextlabel("durationLabel", "duration", LEFT_MARGIN-47, 276);
  
  controlP5.addTextlabel("1", "1", (LEFT_MARGIN + width-200)+10, 50);
  controlP5.addTextlabel("0", "0", (LEFT_MARGIN + width-200)+10, 123);
  controlP5.addTextlabel("-1", "-1", (LEFT_MARGIN + width-200)+8, 193);
  

  // loop through to create the pattern-specific bits of the system
  // since for stereo and morphs we need multiples of all of this
  for(int i=0; i < NUM_PATTERNS; i++) {
    // the fractal model
    fract[i] = new FloatFract();

    // bank of vertical pattern sliders that can work as a "draw" area
    patternSliders[i] = new FWSliderPool(round(stepsSlider.value()), MAX_SLIDERS, LEFT_MARGIN, 0, width-200, 0);
    // set initial values for pattern sliders
    float[] vals = { 1, 0.5f, 1 };
    if(i == 0)
      vals[1] = 0.1f;
    for (int j = 0; j < patternSliders[i].size(); j++)
      patternSliders[i].slider(j).setValue(vals[j]);

    // fractal iteration viewer
    fractView[i] = new IterationView(fract[i].getSegments(), 10, 0, 0, width, 0);
    mouseListeners.add(patternSliders[i]);
  }
  
  stereo(1);
}

/**
  * controlP5 event callbacks
  */

public void a(float val) {
  numPatternsActive = 1;
  patternOffset = 0;
  morphing = false;
  setSingleView(0);
}

public void b(float val) {
  numPatternsActive = 1;
  patternOffset = 1;
  morphing = false;
  setSingleView(1);
}

public void stereo(float val) {
  numPatternsActive = 2;
  patternOffset = 0;
  morphing = false;
  setDoubleView();
}

public void morph(float val) {
  numPatternsActive = 2;
  patternOffset = 0;
  morphing = true;
  setDoubleView();
}

public void looping(boolean val) {
  playback.setLooping(val);
}

public void mute(boolean val) {
  playback.setMute(val);
}

public void setSingleView(int num) {
  patternSliders[num].clear();  
  patternSliders[1-num].clear();  

  patternSliders[num].setHeight(160);
  patternSliders[num].setY(50);
  patternSliders[num].show();

  patternSliders[1-num].hide();
  
  fractView[num].setY(height-30);
  fractView[num].setHeight(-230);
  fractView[num].show();

  fractView[1-num].hide();

  playback.waveDirty = true;
  updateFractalSettings();
}

public void setDoubleView() {
  for(int p = 0; p < numPatternsActive; p++) {
    patternSliders[p].clear();
    patternSliders[p].setY(50+(p*(10+150/NUM_PATTERNS)));
    patternSliders[p].setHeight(150/NUM_PATTERNS);
    patternSliders[p].show();

    fractView[p].clear();
    fractView[p].setY(height-150);  
    fractView[p].setHeight((p==0?-1:1)*230/NUM_PATTERNS);
    fractView[p].show();  
  }
  playback.waveDirty = true;
  updateFractalSettings();
}

public void swap(float val) {
  for (int i = 0; i < patternSliders[0].size(); i++) {
    float tempVal = patternSliders[0].slider(i).value();
    patternSliders[0].slider(i).setValue(patternSliders[1].slider(i).value());
    patternSliders[1].slider(i).setValue(tempVal);
  }
  playback.waveDirty = true;
  updateFractalSettings();
}


public void steps(float val) {
  stepsSlider.setValueLabel(""+round(val));
  checkNumSliders();
}

public void duration(float val) {
  // this will get polled in mouseReleased()
}

public void save(float val) {
  doSave();
}


/**
  * applet events
  */

public void stop() {
  Ess.stop();
  super.stop();
}


public void draw() {
  for(int p = 0; p < numPatternsActive; p++)
    patternSliders[p+patternOffset].draw();

  playback.drawPlayhead();
  
  // process any updates
  if(!update)
    return;
  boolean stillIterating = false;    
  for(int p = 0; p < numPatternsActive; p++) {
    int pat = p + patternOffset;
    if(fract[pat].iteration() < targetIteration) {
      fractView[pat].draw();
      fract[pat].iterate();
      fractView[pat].setNextIteration(fract[pat].getSegments());
      stillIterating = true;
    }
  } 
  if (!stillIterating && playback.waveDirty && playback.audioPlaying) {
    playback.stopAudio();
    if (numPatternsActive == 2) {
      playback.writeStereoAudio(fract[0].getSegments(), fract[1].getSegments());      
    } else {
      for(int p = 0; p < numPatternsActive; p++) {
        int pat = p + patternOffset;
        playback.writeAudio(fract[pat].getSegments(), p);
      }
    }
    playback.playAudio();
  }  
}


public void mousePressed() {
  for(int i = 0; i < mouseListeners.size(); i++)
    ((MouseListener)mouseListeners.get(i)).mousePressed();  
}

public void mouseReleased() {
  for(int i = 0; i < mouseListeners.size(); i++)
    ((MouseListener)mouseListeners.get(i)).mouseReleased();  
  checkNumSliders();
  checkDuration();
  updateFractalSettings();
}



public void dropEvent(DropEvent theDropEvent) {
  if(theDropEvent.isFile()) {
    // for further information see
    // http://java.sun.com/j2se/1.4.2/docs/api/java/io/File.html
    File myFile = theDropEvent.file();
    if(myFile.isFile()) {
      // attempt to parse filename
      String fileName = myFile.getName();
      doRestore(fileName);
    }
  }
}


/*
 * Control Functions
 */ 

public void doSave() {
  playback.stopAudio();
  String defaultFilename = "";
  
  for(int p = 0; p < numPatternsActive; p++) {
    int pat = p + patternOffset;
    
    defaultFilename = "fw_"+fract[pat].toString();
    if(morphing)
      defaultFilename += "->"+fract[1-pat].toString();
      
    if(defaultFilename.length() > 250) {
      defaultFilename = defaultFilename.substring(0, 250);
    }
    defaultFilename += ".aif";
  
    String path = "" + defaultFilename;
    print("Save path: " + path + "\n");  
    playback.writeSoundFile(path, pat);
  }
  
  playback.playAudio();
}

public void doRestore(String fileName) {
  if(fileName.endsWith(".aif") && fileName.startsWith("fw_")) {
    // looks OK-ish
    fileName = fileName.substring(3, fileName.length()-4);  // strip pre- and suffix
    //print("restoring seed:" + fileName);
    playback.stopAudio();
    playback.invalidateAudio();

    FloatFract newFract = new FloatFract(fileName);
    // copy seed onto UI sliders
    stepsSlider.setValue(newFract.pattern().size());
    checkNumSliders();
    checkDuration();
    for(int i = 0; i < patternSliders[patternOffset].size(); i++)
      patternSliders[patternOffset].slider(i).setValue(((Double)newFract.pattern().get(i)).floatValue());
    updateFractalSettings();
    playback.playAudio();
  }
}

public void checkNumSliders() {
  for(int p = 0; p < NUM_PATTERNS; p++) {
    if(patternSliders[p].size() != round(stepsSlider.value())) {
      playback.stopAudio();    
      patternSliders[p].setSize((round(stepsSlider.value())));
      Runtime.getRuntime().gc(); 
    }
  }
}  

public void checkDuration() {
  if(curDuration != durationSlider.value()) {
    curDuration = durationSlider.value();
    if(targetIteration != calculateIterationBounds(0))
      playback.waveDirty = true;
  }
}


public void updateFractalSettings() {
  boolean needsUpdate = false;
  boolean waveWasDirty = playback.waveDirty;
  ArrayList[] newPattern = new ArrayList[numPatternsActive];
  for(int p = 0; p < numPatternsActive; p++) {
    if(!morphing)
      fract[p+patternOffset].setMorphPattern(null);

    newPattern[p] = new ArrayList(patternSliders[p+patternOffset].size());
    for(int i = 0; i < patternSliders[p+patternOffset].size(); i++) {
      newPattern[p].add(new Double(patternSliders[p+patternOffset].slider(i).value()));
    }
    if(waveWasDirty || (!patternsSame(newPattern[p], fract[p+patternOffset].pattern()))) {
      needsUpdate = true;
    }
  }
  
  if(needsUpdate) {
    targetIteration = calculateIterationBounds(0);
    for(int p = 0; p < numPatternsActive; p++) {
      fract[p+patternOffset].setPattern(newPattern[p]); 
      if(morphing) {
        fract[p+patternOffset].setMorphPattern(newPattern[1-p]); 
      }
      fractView[p+patternOffset].reset(this);
      fractView[p+patternOffset].setNextIteration(fract[p+patternOffset].getSegments());
    }
    playback.invalidateAudio();
    Runtime.getRuntime().gc(); 
    playback.playAudio();    
  }
}

public int calculateIterationBounds(int p) { 
  // numSliders^iteration = total samples
  float targetLength = durationSlider.value();
  float numIterations = log(SR*targetLength) / log(patternSliders[p].size());
  
  int targetIteration = ceil(numIterations); // favor longer clips
  // unless it would be too long
  if (pow(patternSliders[p].size(), targetIteration) >= SR*targetLength*4)
    targetIteration--;
    
  return targetIteration;
}


public class FWAudio {
 
  public AudioChannel[] buffers = new AudioChannel[2];
  public boolean waveDirty = false;
  public boolean audioPlaying = false;
  public boolean looping = true;
 
  public FWAudio() {
  }
  
  public void writeStereoAudio(ArrayList l, ArrayList r) {
    writeAudio(l, 0);
    writeAudio(r, 1);
    buffers[0].pan(Ess.LEFT);
    buffers[1].pan(Ess.RIGHT);
  }
  
  public void writeAudio(ArrayList samples, int bufferNumber) {
    int numSamples = samples.size();

    AudioChannel mySound = buffers[bufferNumber] = new AudioChannel();    
    mySound.initChannel(numSamples);

    Double curVal;
  /*
    // find peak to normalize audio
    double peak = 0.01;
    double dVal;
    for(int i = 0; i < numSamples; i++) {
      curVal = (Double)samples.get(i);
      dVal = curVal.doubleValue();
      if (dVal > peak)
        peak = dVal;
    }    
  */

    // grab sample values from the fractal
    float[] buffer = new float[numSamples];
    for(int i = 0; i < numSamples; i++) {
      curVal = (Double)samples.get(i);
      buffer[i] = curVal.floatValue(); // /(float)peak;
    }

    mySound.samples = buffer;
    mySound.pan(0);
    waveDirty = false;
  }

  public void writeSoundFile(String filename, int bufferNumber) {
     AudioFile myFile = new AudioFile();
     myFile.open(filename, (float)SR, Ess.WRITE);   
     myFile.write(buffers[bufferNumber].samples);
     myFile.close();   
  }

  // sound buffer can be invalid, but the system
  // is still trying to play. this basically says
  // we need to regenerate the audio
  public void invalidateAudio() {
    stopBuffers();
    buffers[0] = buffers[1] = null;
    waveDirty = true;
  }  

  public void stopAudio() {
    stopBuffers();
    audioPlaying = false;
  }

  public void playAudio() {
    audioPlaying = true;
  
    if(waveDirty) return;
  
    for(int i = 0; i < buffers.length; i++) {
      if (buffers[i] != null) {
        buffers[i].stop();
        buffers[i].play(looping ? Ess.FOREVER : 1); 
      }
    }
  }

  public void setLooping(boolean v) {
    if((v && !looping) || (!v && looping)) {
      this.looping = v;
      stopBuffers();
      playAudio();
    }
  }

  public void setMute(boolean v) {
    for(int i = 0; i < buffers.length; i++) {
      if (buffers[i] != null) {
        buffers[i].mute(v);
      }
    }
  }

  public void drawPlayhead() {
    for(int i = 0; i < buffers.length; i++) {
      if (buffers[i] != null) {
        fill(0);
        stroke(0);
        rect(0, height, width, -20);
        float phase = buffers[i].cue * 1.0f / buffers[i].size;
        stroke(0.25f, 0.8f, 1);
        line(width*phase, height, width*phase, height-19);
      }
    }
  }
  
  private void stopBuffers() {
    for(int i = 0; i < buffers.length; i++) {
      if (buffers[i] != null)
        buffers[i].stop(); 
    }    
  }
  
}
class FWSlider {
  float x, y;
  float w, h;
  float value;
  float minVal;
  float maxVal;
  
  float range;
  
  boolean logarithmic;
  
  FWSliderPool parent;
  
  public FWSlider(FWSliderPool parent) {
    this.parent = parent;
  }
  
  public void setRange(float minVal, float maxVal) {
    this.minVal = minVal;
    this.maxVal = maxVal;
    this.range = maxVal - minVal;
  }
  
  public void setSize(float x, float y, float w, float h) {
    this.x = x;
    this.y = y;
    this.w = w;
    this.h = h;
  }
  
  public void setValue(float v) { this.value = v; } 
  public float value() { return this.value; }
  
  // not implemented
  public void setLogarithmic() { this.logarithmic = true; }
  
  public void draw() {
      if(parent.lock && mousePressed) {
        this.updatePosition();
      }
      
      float curValIntensity = abs(this.value);
      float belowZero = (this.value < 0.0f ? 1.0f : 0.0f);
      int c = color(0.0f, belowZero, curValIntensity);
      stroke(0.35f); // dark outline
      fill(c);
      rect(this.x-1, this.y-1, this.w+2, this.h+2);  // border
      
      stroke(0.25f);
      line(this.x, this.y+this.h/2.0f, this.x+this.w, this.y+this.h/2.0f); // middle detent
      
      stroke(0.5f, 0.8f, 0.65f);
      fill(0.5f, 0.8f, 0.65f);
      rect(this.x, this.y + (this.h*(this.maxVal - this.value)/(this.maxVal - this.minVal)), this.w, 1.0f); // tick mark
  }
  
  public boolean mouseOver() {
    // bounds
    int slop = 3;
    if (mouseX >= this.x && mouseX <= this.x + this.w && mouseY >= this.y - slop && mouseY <= this.y + this.h + slop) {
      return true;
    } else {
      return false;
    }    
  }
  
  public void updatePosition() {
    // bounds
    int slop = 3;
    if (this.mouseOver()) {
      this.value = ((this.y + this.h) - mouseY)/this.h;
      this.value = min(1, max(0, this.value)); // limit
      this.value *= this.range;
      this.value += this.minVal;
    }
  }
}
class FWSliderPool implements MouseListener {
  protected FWSlider[] sliders;
  protected int size;
  
  boolean lock; // for click tracking
  
  boolean visible;
  
  public int x, y;
  public int w, h;

  public FWSliderPool(int numSliders, int maxSliders, int x, int y, int w, int h) {
    this.x = x;
    this.y = y;
    this.w = w;
    this.h = h;
    
    this.visible = true;
    this.lock = false;
    
    // pre-allocate pool of pattern sliders
    sliders = new FWSlider[maxSliders];
    for (int i = 0; i < maxSliders; i++) {
      sliders[i] = new FWSlider(this);
      sliders[i].setRange(-1, 1);
    }
    
    setSize(numSliders);
  }

  public FWSlider slider(int index) {
    return sliders[index];
  }

  public void mouseDragged() {}
  
  public void mousePressed() {
    if(this.visible && this.mouseOver())
      this.lock = true;
  }
  
  public void mouseReleased() {
    this.lock = false;
  }
  
  public void hide() { this.visible = false; }
  public void show() { this.visible = true; }

  public int size() { return size; }

  public void setSize(int size) {    
    this.size = size;
    float sliderWidth = w*1.0f/size;
    int i;
    for(i = 0; i < size; i++) {
      sliders[i].setSize(x+i*sliderWidth, y, sliderWidth, h);
      sliders[i].setValue(0);
    }
  
    // clear remaining
    for(i = i; i < sliders.length; i++) {
      sliders[i].setValue(0);
    }
  }
  
  public void setY(int y) {    
    this.y = y;
    for(int i = 0; i < size; i++) {
      sliders[i].y = y;
    }
  }

  public void setHeight(int h) {
    this.h = h;
    for(int i = 0; i < size; i++) {
      sliders[i].h = h;
    }    
  }
  
  public void clear() {
    stroke(0);
    fill(0);
    rect(x,y,w,h);
  }
  
  public void draw() {
    if(!this.visible)
      return;
      
    clear();
    
    for(int i = 0; i < size; i++)
      sliders[i].draw();      
  }
  
  
  public boolean mouseOver() {
    // bounds
    int slop = 3;
    if (mouseX >= this.x && mouseX <= this.x + this.w && mouseY >= this.y - slop && mouseY <= this.y + this.h + slop) {
      return true;
    } else {
      return false;
    }    
  }
  
}

// Generalization of the Cantor Set.  
//
// Uses an array of real-valued coefficients as a "pattern" to recursively
// sub-divide.  
//
class FloatFract {
  ArrayList pattern;
  ArrayList morphPattern;
  ArrayList segments;
  int iteration;

  public FloatFract() {
    pattern = new ArrayList();     
    reset();
  }

  // way to reconstruct a fractal from serialized string as given by toString() method
  public FloatFract(String s) {
    pattern = new ArrayList();     
    reset();
    for(int i = 0; i < s.length()/2; i++) {
      String hex = s.substring(2*i, 2*i+2);
      int intValue = Integer.parseInt(hex, 16);
      double floatValue = (intValue/255.0f)*2.0f-1.0f;
      pattern.add(new Double(floatValue));
    }
  }

  public void reset() {
    iteration = 0;
    segments = new ArrayList();
    segments.add(new Double(1.0f));
  }

  public void setPattern(ArrayList c) { 
    reset(); 
    pattern = c; 
  }  
  
  public void setMorphPattern(ArrayList m) {
    morphPattern = m;
  }

  public ArrayList pattern() { return this.pattern; }
  
  public ArrayList getSegments() { 
    return segments; 
  }

  public void iterate() {
    if (morphPattern != null) {
      iterateMorph();
      return;
    }
      
    ArrayList newSegments = new ArrayList();
    for(int i = 0; i < segments.size(); i++) {
      Double oldVal = (Double)segments.get(i);
      for(int j = 0; j < pattern.size(); j++) {
        newSegments.add((Double)pattern.get(j) * oldVal);
      }
    }
    segments = newSegments;    
    iteration++;
  }

  public void iterateMorph() {
    ArrayList newSegments = new ArrayList();
    double phase = 0;
    double delta = 1.0f/((segments.size())*(pattern.size())-1.0f);
    double lerpedWeight = 0.0f;
    for(int i = 0; i < segments.size(); i++) {
      Double oldVal = (Double)segments.get(i);
      for(int j = 0; j < pattern.size(); j++) {
        Double patternVal = (Double)pattern.get(j);
        Double morphPatternVal = (Double)morphPattern.get(j);
        lerpedWeight = (1.0f-phase)*(patternVal) + (phase)*(morphPatternVal);
        newSegments.add(lerpedWeight * oldVal);
        phase += delta;
      }
    }
    segments = newSegments;    
    iteration++;
  }


  public int iteration() { 
    return iteration; 
  }
  
  public int sizeAfterNthIteration(int n) {
    return round(pow(pattern.size(), n));
  }
  
  // writes seed out to string.  
  // each element in the pattern is represented as two chars hexidecimal.
  public String toString() {
    String name = "";
    String hex = "";
    for(int i = 0; i < pattern.size(); i++) {
      Double v = (Double)pattern.get(i);
      Integer intVal = floor((float)(255*(v+1.0f)/2.0f));
      hex = Integer.toHexString(intVal);
      if(hex.length() < 2) {
        hex = "0" + hex;
      } else if (hex.length() > 2) {
        hex = "ff";
      }
      name += hex; 
    }
    return name;
  }
}


public boolean patternsSame(ArrayList a, ArrayList b) {
   if(a.size() != b.size()) return false;   
   for(int i = 0; i < a.size(); i++) {
     if(((Double)a.get(i)).doubleValue() != ((Double)b.get(i)).doubleValue()) return false;
   }

   return true;
}

class IterationView {
  public int x, y, w, h;
  public int maxIterationsShown;
  public int curIteration;
  
  boolean visible;
  
  private static final int OVERSAMPLE = 32;

  public ArrayList segments;
  
  public IterationView(ArrayList initialSegments, int maxIterationsShown, int x, int y, int w, int h) {
    segments = initialSegments;
    curIteration = 0;
    this.maxIterationsShown = maxIterationsShown;
    this.x = x;
    this.y = y;
    this.w = w;
    this.h = h;
    
    this.visible = true;
  }
  
  public void setNextIteration(ArrayList segments) { 
    this.segments = segments; 
    this.curIteration++; 
  }
  
  
  public void hide() { this.visible = false; }
  public void show() { this.visible = true; }

  public void setHeight(int h) { this.h = h; }
  public void setY(int y) { this.y = y; }

  public void draw() {
    if (!this.visible) return;
    if (segments == null) return;
    
    float yStep = h / maxIterationsShown;
    
    if (segments.size() < OVERSAMPLE*w && curIteration < maxIterationsShown) {
      int numSegments = segments.size();
      for(int i = 0; i < numSegments; i++) {
        Double curVal = (Double)segments.get(i);
        float curValIntensity = abs(curVal.floatValue());
        float belowZero = (curVal < 0.0f ? 1.0f : 0.0f);
        int c = color(0.0f, belowZero, curValIntensity);
        stroke(c);
        fill(c);
        rect(x+w*i/numSegments, y+curIteration*yStep, w/numSegments, (yStep*0.8f));
      }  
    }
  }

  public void clear() {
    fill(0);
    stroke(0);
    rect(x, y, w, h);   
  }

  public void reset(PApplet theApplet) {
    curIteration = 0;
    segments = null;
    clear();
   }
}
public interface MouseListener {
  public void mousePressed();
	public void mouseReleased();
	public void mouseDragged();
}

  static public void main(String args[]) {
    PApplet.main(new String[] { "--bgcolor=#c0c0c0", "FractalWavetables" });
  }
}
